
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import static java.util.concurrent.Executors.newFixedThreadPool;

public class ProfilingList {

// test results on laptop i7 processor (two cores with hyperthreading)    
// Threads 1 with time: 13877
// Threads 2 with time: 8525
// Threads 3 with time: 7119
// Threads 4 with time: 6691
// Threads 5 with time: 6136
// Threads 10 with time: 12014
    
    public static void main(String[] args) {

        ProfilingList tP = new ProfilingList();
        List<String> results = new ArrayList<>();

        results.add(tP.test(1));
        results.add(tP.test(2));
        results.add(tP.test(3));
        results.add(tP.test(4));
        results.add(tP.test(5));
        results.add(tP.test(10));

        for (String result : results) {
            System.out.println(result);
        }

    }

    ExecutorService threadPool;

    public String test(int numberOfThreads) {

        threadPool = newFixedThreadPool(numberOfThreads);
        List<Future> futures = new ArrayList<>();
        long time = System.currentTimeMillis();

        for (int i = 1; i <= 10; i++) {
            Callable<String> aCallable = new CallableImpl(i);
            Future<String> submit = threadPool.submit(aCallable);
            futures.add(submit);
        }

        try {
            for (Future future : futures) {
                System.out.println(future.get());
            }
        } catch (Exception ex) {
            System.out.println("Not ready ?? ...");
        }

        threadPool.shutdown(); // shutdown the pool.

        return ("Threads " + numberOfThreads + " with time: " + (System.currentTimeMillis() - time));

    }

    private class CallableImpl implements Callable<String> {

        String name;

        public CallableImpl(int number) {
            name = "Callable " + number;
        }

        @Override
        public String call() {
            Random random = new Random();
            for (int i = 0; i < 2; i++) {
                System.out.println(name + " is doing something");
                List<Double> arr = new LinkedList();
                for (int j = 0; j < 1000000; j++) {
                    arr.add(random.nextDouble());
                }
                System.out.println(name + " [" + i + "] Max: " + getMax(arr));
                System.out.println(name + " [" + i + "] Min: " + getMin(arr));
                System.out.println(name + " [" + i + "] Avg: " + getAvg(arr));
                System.out.println(name + " [" + i + "] Sum: " + getSum(arr));
                System.out.println(name + " [" + i + "] Med: " + getMed(arr));
            }
            return name + " is done !";
        }

        private Double getMax(List<Double> arr) {
            return Collections.max(arr);
        }

        private Double getMin(List<Double> arr) {
            return Collections.min(arr);
        }

        private Double getSum(List<Double> arr) {
            double sum = 0;
            for (Double d : arr) {
                sum += d;
            }
            return sum;
        }

        private Double getAvg(List<Double> arr) {
            return getSum(arr) / arr.size();
        }

        private Double getMed(List<Double> arr) {
            Collections.sort(arr);
            return arr.get(arr.size() / 2);
        }

    }

}
